// This may look like C code, but it is really -*- C++ -*-
/*
 *************************************************************************
 *
 *			  Basic Window class for XVT
 *
 *	   A generic simple window with controls/subwindows (if any)
 *    that can be dragged around on the screen and closed by clicking a
 *			      "go-away" button
 *
 * Class XVTBasicWindow provides a very generic event and control processing
 * that is common to regular windows with controls, modal and modeless
 * dialogs. The basic functionality is to route events to appropriate
 * handlers, process close/destroy/etc events, and handle controls.
 * The class XVTBasicWindow provides only basic event handling, and it's
 * abstract: you have to create your own class on the base of XVTBasicWindow
 * and define the draw() function that is called on Update event to
 * paint/repaint the window. You may want to redefine couple of other
 * vanilla handlers (say, for control items (buttons, etc)).
 * XVTBasicWindow does not have a publicly available constructor, since
 * construction of these windows really depends on the type of window/dialog.
 * So, the only way to use XVTBasicWindow is to derive a subclass on its
 * base. The subclass should call create_res_window() or create_res_dialog()
 * or the likes and use (EVENT_HANDLER)q_universal_handler() as the event
 * handler and (long)this as an object RefCon. Unless the modal dialog is
 * created, the derived class should define draw() function (and control
 * handlers if appropriate).
 * Again, unless a modal dialog has been created, serve_to_death()
 * function should be called to process events (for the newly created
 * and other windows).
 *
 * Class XVTWindow is declared on the base of XVTBasicWindow to create
 * a window with controls/subwindows (if any). You can create as many
 * windows as you want. When you create the window using explicit
 * specification of the rectangle, title, etc, the window
 * would be invisible right after the creation. It's better to set up
 * invisible (or hidden) attribute in the window resource if you create
 * a window from the resource template. That way you still can do some
 * rearrangements (say, create child windows) before the event processing
 * starts.
 *
 * To show the window (with all the controls, children etc) and start
 * processing of events, call show(). If you need to wait until the window
 * is closed (after the user clicked some button or clicked at the go-away
 * region, or selected "Close" from the menu, or something) call the
 * serve_to_death() function. It returns only when all the opened windows
 * are closed.
 * XVTWindow does not define draw() function, so it's also abstract.
 * To use XVTWindow, one has to create his own class on its base, define
 * draw(), and, if necessary, redefine event handler/control handlers, etc.
 *
 * $Id: Window.h,v 2.2 1994/11/07 20:33:30 oleg Exp oleg $
 *
 *************************************************************************
 */
 
#ifndef __GNUC__
#pragma once
#else
#pragma interface
#endif

#ifndef _Window_h
#define _Window_h 1

#include "xvt-serv.h"

class XVTBasicWindow;
class IMAGE;

class XVTRect : public RCT
{
  XVTRect(void) {}		// supposed to be filled in later
public:

  XVTRect(const XVTBasicWindow& xvt_win);
  XVTRect(const IMAGE& image);
  ~XVTRect(void) {}
  XVTRect& operator += (const int offset);
  operator RCT * (void) 		{ return this; }
	
  int q_width(void)  const 		{ return abs(right-left); }
  int q_height(void) const		{ return abs(bottom-top); }
				// Give a STATIC string representation of
				// the rectangle
  operator const char * (void) const;
};

class XVTBasicWindow
{
  friend class XVTRect;

  WINDOW this_window;
  BOOLEAN Not_canceled;		// set to FALSE if Cancel button was pressed
				// or the window was forcibly closed

#ifdef __SC__
				// Variable number of arguments is just to
				// force the C calling convention. Normally
				// member functions are called using the
				// pascal conventions in Symantec C++
  static long universal_handler(WINDOW win, EVENT *ep,...);
#else
  static long universal_handler(WINDOW win, EVENT *ep);
#endif
  static int no_opened_windows;
	
				// It is this function that really draws smth
  virtual void draw(void) = 0;

protected:
  XVTBasicWindow(void);		// Protected constructor
			// Universal handler is called by XVT only.
			// Real window constructors need only a pointer to
			// that function to pass on to create_window() or
			// its likes. That's why the function pointer
			// is returned as void *
  void * q_universal_handler(void) const    { return (void*)universal_handler;}

  virtual void handler(EVENT event);	// More specific handler
			// Initialize the Window, return FALSE to cancel
  virtual BOOLEAN initialize(void);
			// Return TRUE to continue, FALSE to Quit
  virtual BOOLEAN handle_ctl(const short id, const CONTROL_INFO& ctl);

  const WINDOW q_window(void) const	{ return this_window; }
			// Return a window associated with a
			// specific control/child window
  const WINDOW q_window(const short id) const;

			// Set a title of a control item (say, button, etc)
  void set_title(const short id, const char * text);
  const char * get_title(const short id) const;
  void enable(const short id, const BOOLEAN onoff);
  void show(const short id, const BOOLEAN onoff);
  void set_checked(const short id,const BOOLEAN onoff);	// Set checked button

			// Functions specific to List Controls
				// Get the index of the 1st selected item
  int count_selections(const short id) const;
  int get_selection(const short id) const;
  void set_selection(const short id, const short item_index,
		     const BOOLEAN onoff=TRUE);
  void add_to_list_control(const short id, const char * str);
  void add_list(const short id, const ListofStrings& list);
  void clear_list(const short id);  // Remove all items from the list control

				// Draw w/o waiting for UPDATE event:
				// Use for animation 
  void draw_immediately(void)	{ draw(); }

				// Indicate that a long operation is
				// in progress
  void set_cursor_wait(const BOOLEAN onoff);

public:
  virtual ~XVTBasicWindow(void);
				// Serve opened windows until all get closed 
  static void serve_to_death(void);

				// Call this to start the service
  void show(void) const		{ ::show_window(this_window, TRUE); }
  void show(const BOOLEAN onoff) const	{ ::show_window(this_window, onoff); }
  virtual void cancel(void);		// Close the window
  operator BOOLEAN (void) const		{ return Not_canceled; }
};

class XVTWindow : public XVTBasicWindow
{
protected:
  XVTWindow(void) {}		// Call appropriate creator to open the window

public:
  XVTWindow(XVTRect rect, const char * title)
  			{ create(rect, title); }
  XVTWindow(const short resource_id, const WINDOW parent_win=SCREEN_WIN)
  			{ create(resource_id, parent_win); }
  ~XVTWindow(void) {}
				// Use these functions to create windows.
				// Event processing may start *before* the
				// functions return!
  void create(XVTRect rect, const char * title);
  void create(const short resource_id, const WINDOW parent_win=SCREEN_WIN);
			// Create a child window laying over a parent window
  void create(const XVTBasicWindow& parent);
};


inline XVTRect::XVTRect(const XVTBasicWindow& xvt_win)
  	{ get_client_rect(xvt_win.q_window(),(RCT *)this); }

#endif
