// This may look like C code, but it is really -*- C++ -*-
/*
 *************************************************************************
 *
 *				Rubber rectangle
 *
 * A rectangular outline that can be stretched, squeezed and dragged
 * around within an image window. The rubber rectangle is used to specify
 * a rectangular subarea of the image.
 *
 * Once rectangle is displayed, the user can stretch/squeeze it.
 * Rectangle disappears when somebody requests it. This is consistent
 * with the way, say, drawing programs work. You can pick a "rectangular
 * selection" tool (usually by clicking on an appropriate icon in the
 * tool palette) and start dragging/stretching the rectangle over the
 * picture. Then you can select an appropriate action (from a menu,
 * using the keyboard shortcut, etc) to, say, invert the colors within
 * the rectangle, zoom in on the rectangle, group items within the rectangle,
 * etc. Note that executing the action dismisses the rubber rectangle.
 * If one changes his mind about the using of a rubber rectangle, he
 * can pick another tool (say, arrow (pointer) tool), and the rubber
 * rectangle disappears.
 *
 * If the mouse held down within the rubber rectangle, the rectangle
 * moves with the mouse. If the mouse is held down exactly on the left/
 * right border of the rectangle, moving the mouse moves that border
 * left and right. The same holds for the top/bottom border of the rectangle.
 * Clicking right on the corner of the rectangle and dragging changes
 * both dimensions of the rectangle. In every case, the rectangle
 * always remains within the picture.
 *
 * Note, we want to have this rubber rectangle over an existing window;
 * and we want to keep the window even after the rectangle is dismissed.
 * Obviously this means we can't base the rubber rectangle class on
 * the XVTImageWindow. Indeed, we don't want to get into trouble of
 * switching event processing between the rubber rectangle (when it's
 * shown) and that of the regular image window. So, we make a special
 * transparent child window laying over the image window (just over
 * the internal part of the window, that is, over a picture itself;
 * menu, border, etc are not included). That way, we get our own event
 * processing plus we can create/close the rubber rectangle's window any
 * time, without affecting the parent image window. Note, that somebody
 * got to call 'serve_to_death()' somewhere to dispatch events.
 *
 * $Id: rubber_rect.h,v 1.1 1994/11/10 19:16:25 oleg Exp oleg $
 *
 *************************************************************************
 */

#ifndef __GNUC__
#pragma once
#else
#pragma interface
#endif

#include "image_win.h"

class RubberRectangle : protected XVTWindow
{
  XVTRect rect;				// we guarantee rect.left < rect.right
					// and rect.top < rect.bottom
  const XVTImageWindow& parent;

  enum MOUSE_POS { MT_none=0, MT_in_rect=1,
		   MT_top=2, MT_bottom=4, MT_left=8, MT_right=16 }
  		mouse_touchdown_type;	// If mouse is clicked on the upperleft
					// corner, MT_top|MT_left are set
  PNT mouse_pos;			// Reference mouse position
					// (when watch_mouse is not MT_none)
  MOUSE_POS classify_pos(const PNT pt);	// based on pt position

				// It is this function that draws the rect
  virtual void draw(void);

protected:
				// Called whenever the rectangle is changed
				// or moved. Return FALSE to kill the 
				// RubberRectangle
  virtual BOOLEAN processed_changed_rect(const XVTRect& rect) const;

  virtual void handler(EVENT event);	// More specific handler

  BOOLEAN handle_key_down(const short key_code, 
			  BOOLEAN shift_pressed, BOOLEAN ctlr_pressed);
	
				// Handle the mouse movement from 'mouse_pos'
				// to 'new_point'
  void handle_mouse_move(const PNT new_point);

			// Initialize the Window, return FALSE to cancel
  virtual BOOLEAN initialize(void);

public:
  RubberRectangle(const XVTImageWindow& parent);
  virtual ~RubberRectangle() {}
  void init_rectangle(void);		// Make an initial rectangle
  operator BOOLEAN (void) const		// Is rectangle shown?
		{ return XVTBasicWindow::operator BOOLEAN (); }
  void open(void);			// Show the rectangle
  virtual void cancel(void);		// Dismiss the rubber rectangle,
					// but RubberRectangle::rect is valid
  const XVTRect& q_rect(void) const	{ return rect; }
  void draw_immediately(void);
};
